
import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;

public class MainFrame extends JFrame{
	
	private static final String PARSE_ERROR_MSG = "error";
	
	private int width;
	private int height;
	public static final String PRODUCT_NAME = "dec2hex";
	public static final String PRODUCT_VERSION = "v0.1";
	
	private static final int JTF_WIDTH = 100;
	private int arrow_width;
	
	private JTextField jtf_dec;
	private JTextField jtf_hex;
	private JLabel jl_msg;
	private JCheckBox jcb;
	private ContextMenu cm;
	
	public MainFrame(){
		setTitle(PRODUCT_NAME + " " + PRODUCT_VERSION);
		setDefaultCloseOperation(EXIT_ON_CLOSE);
		Toolkit tk = this.getToolkit();
		Dimension screen_size = tk.getScreenSize();
		setResizable(false);
		init_components();
		// show context menu
		addMouseListener(
			new MouseAdapter(){
				public void mouseReleased(MouseEvent e){
					if(!SwingUtilities.isRightMouseButton(e))
						return;
					cm.show(e.getComponent(), e.getX(), e.getY());
				}
			}
		);
		
		setVisible(true);
		// 先にsetVisible(true)しないとダメ
		Insets ins = getInsets();
		setBounds(
			(screen_size.width - width) / 2,
			(screen_size.height - height) / 2,
			width + ins.left + ins.right,
			height + ins.top + ins.bottom
		);
	}
	
	private void init_components(){
		Container con = getContentPane();
		con.setLayout(null);
		// initialize labels
		JLabel jl_dec = new JLabel("Dec");
		JLabel jl_hex = new JLabel("Hex");
		JLabel jl_arrow = new JLabel("⇔");
		jl_msg = new JLabel(PRODUCT_NAME);
		Dimension dim_jl = jl_dec.getPreferredSize();
		Dimension dim_jl_msg = jl_msg.getPreferredSize();
		arrow_width = jl_arrow.getPreferredSize().width;
		
		//initialize textfields
		jtf_dec = new JTextField("0");
		jtf_hex = new JTextField("0");
		Dimension dim_jtf = jtf_dec.getPreferredSize();
		jtf_dec.addKeyListener(
			new KeyListener(){
				public void keyPressed  (KeyEvent e) {change_hex();}
				public void keyReleased (KeyEvent e) {change_hex();}
				public void keyTyped    (KeyEvent e) {change_hex();}
			}
		);
		jtf_hex.addKeyListener(
			new KeyListener(){
				public void keyPressed  (KeyEvent e) {change_dec();}
				public void keyReleased (KeyEvent e) {change_dec();}
				public void keyTyped    (KeyEvent e) {change_dec();}
			}
		);
		
		jcb = new JCheckBox("Always on top");
		Dimension dim_jcb = jcb.getPreferredSize();
		jcb.addChangeListener(
			new ChangeListener(){
				public void stateChanged(ChangeEvent e){
					change_ontop();
				}
			}
		);
		
		cm = new ContextMenu(this);
		
		// set window width/height
		width = 2 * JTF_WIDTH + arrow_width;
		height = dim_jl.height + dim_jtf.height + dim_jl_msg.height;
		
		// add labels
		con.add(jl_dec);
		con.add(jl_hex);
		con.add(jl_arrow);
		con.add(jl_msg);
		
		// add textfields
		con.add(jtf_dec);
		con.add(jtf_hex);
		
		con.add(jcb);
		
		con.add(cm);
		
		// move labels
		jl_dec.setBounds(0, 0, dim_jl.width, dim_jl.height);
		jl_hex.setBounds(JTF_WIDTH + arrow_width, 0, dim_jl.width, dim_jl.height);
		jl_arrow.setBounds(JTF_WIDTH, dim_jl.height, arrow_width, dim_jl.height);
		jl_msg.setBounds(0, dim_jl.height + dim_jtf.height, width - dim_jcb.width, dim_jl_msg.height);
		
		// move textfields
		jtf_dec.setBounds(0, dim_jl.height, JTF_WIDTH, dim_jtf.height);
		jtf_hex.setBounds(JTF_WIDTH + arrow_width, dim_jl.height, JTF_WIDTH, dim_jtf.height);
		
		jcb.setBounds(width - dim_jcb.width, dim_jl.height + dim_jtf.height, dim_jcb.width, dim_jl_msg.height);
		
		
	}
	
	public void change_dec(){
		long n = 0;
		try{
			n = Long.valueOf(jtf_hex.getText(), 16);
			show_msg(to_little_endian(n));
		}catch(NumberFormatException e){
			show_msg(PARSE_ERROR_MSG);
		}
		jtf_dec.setText(Long.toString(n, 10));
	}
	
	public void change_hex(){
		long n = 0;
		try{
			n = Long.valueOf(jtf_dec.getText(), 10);
			show_msg(to_little_endian(n));
		}catch(NumberFormatException e){
			show_msg(PARSE_ERROR_MSG);
		}
		jtf_hex.setText(Long.toString(n, 16).toUpperCase());
	}
	
	public void change_ontop(){
		setAlwaysOnTop(jcb.isSelected());
	}
	
	public void show_msg(String msg){
		jl_msg.setText(msg);
	}
	
	public static String to_little_endian(long n){
		String s = Long.toString(n, 16);
		int l = s.length();
		if(l % 2 == 1){
			s = "0" + s;
			l++;
		}
		String result = new String();
		int i;
		for(i=0; i<l/2; i++){
			if(i != 0)
				result += " ";
			result += s.substring(l- i * 2 - 2, l - i * 2);
		}
		return result.toUpperCase();
	}
	
}

class ContextMenu extends JPopupMenu{
	
	private MainFrame mf;
	
	public ContextMenu(MainFrame f){
		this.mf = f;
		// initialize
		JMenuItem jmi_about = new JMenuItem("About...");
		final JPanel jp_about = new JPanel();
		jp_about.setLayout(new BoxLayout(jp_about, BoxLayout.Y_AXIS));
		jp_about.add(new JLabel(MainFrame.PRODUCT_NAME + " " + MainFrame.PRODUCT_VERSION));
		jp_about.add(new JLabel("(c) epii 2008"));
		jp_about.add(new HyperLinkLabel("http://epii.homelinux.net"));
		jmi_about.addActionListener(
			new ActionListener(){
				public void actionPerformed(ActionEvent e){
					JOptionPane.showMessageDialog(
						mf,
						jp_about,
						"about",
						JOptionPane.INFORMATION_MESSAGE
					);
				}
			}
		);
		add(jmi_about);
	}
}

